@@ -22,6 +22,11 @@
 -- aren't truly arbitrary precision. I intend to change the type of the exponent
 -- to 'Integer' in a future release.
 --
+-- /WARNING:/ Although @Scientific@ has instances for all numeric classes the
+-- methods should be used with caution when applied to scientific numbers coming
+-- from untrusted sources. See the warnings of the instances belonging to
+-- 'Scientific'.
+--
 -- The main application of 'Scientific' is to be used as the target of parsing
 -- arbitrary precision numbers coming from an untrusted source. The advantages
 -- over using 'Rational' for this are that:
@@ -41,13 +46,6 @@
 -- to @'Integral's@ (like: 'Int') or @'RealFloat's@ (like: 'Double' or 'Float')
 -- will always be bounded by the target type.
 --
--- /WARNING:/ Although @Scientific@ is an instance of 'Fractional', the methods
--- are only partially defined! Specifically 'recip' and '/' will diverge
--- (i.e. loop and consume all space) when their outputs have an infinite decimal
--- expansion. 'fromRational' will diverge when the input 'Rational' has an
--- infinite decimal expansion. Consider using 'fromRationalRepetend' for these
--- rationals which will detect the repetition and indicate where it starts.
---
 -- This module is designed to be imported qualified:
 --
 -- @import Data.Scientific as Scientific@
@@ -66,8 +64,14 @@ module Data.Scientific
     , isInteger
 
       -- * Conversions
+      -- ** Rational
+    , unsafeFromRational
     , fromRationalRepetend
+    , fromRationalRepetendLimited
+    , fromRationalRepetendUnlimited
     , toRationalRepetend
+
+      -- ** Floating & integer
     , floatingOrInteger
     , toRealFloat
     , toBoundedRealFloat
@@ -99,7 +103,6 @@ import           Control.DeepSeq              (NFData, rnf)
 import           Data.Binary                  (Binary, get, put)
 import           Data.Char                    (intToDigit, ord)
 import           Data.Data                    (Data)
-import           Data.Function                (on)
 import           Data.Hashable                (Hashable(..))
 import           Data.Int                     (Int8, Int16, Int32, Int64)
 import qualified Data.Map            as M     (Map, empty, insert, lookup)
@@ -116,6 +119,10 @@ import qualified Text.ParserCombinators.ReadP    as ReadP
 import           Text.ParserCombinators.ReadP     ( ReadP )
 import           Data.Text.Lazy.Builder.RealFloat (FPFormat(..))
 
+#if !MIN_VERSION_base(4,9,0)
+import           Control.Applicative          ((*>))
+#endif
+
 #if !MIN_VERSION_base(4,8,0)
 import           Data.Functor                 ((<$>))
 import           Data.Word                    (Word)
@@ -174,42 +181,63 @@ scientific = Scientific
 instance NFData Scientific where
     rnf (Scientific _ _) = ()
 
+-- | A hash can be safely calculated from a @Scientific@. No magnitude @10^e@ is
+-- calculated so there's no risk of a blowup in space or time when hashing
+-- scientific numbers coming from untrusted sources.
 instance Hashable Scientific where
-    hashWithSalt salt = hashWithSalt salt . toRational
+    hashWithSalt salt s = salt `hashWithSalt` c `hashWithSalt` e
+      where
+        Scientific c e = normalize s
 
+-- | Note that in the future I intend to change the type of the 'base10Exponent'
+-- from @Int@ to @Integer@. To be forward compatible the @Binary@ instance
+-- already encodes the exponent as 'Integer'.
 instance Binary Scientific where
-    put (Scientific c e) = do
-      put c
-      -- In the future I intend to change the type of the base10Exponent e from
-      -- Int to Integer. To support backward compatibility I already convert e
-      -- to Integer here:
-      put $ toInteger e
-
+    put (Scientific c e) = put c *> put (toInteger e)
     get = Scientific <$> get <*> (fromInteger <$> get)
 
+-- | Scientific numbers can be safely compared for equality. No magnitude @10^e@
+-- is calculated so there's no risk of a blowup in space or time when comparing
+-- scientific numbers coming from untrusted sources.
 instance Eq Scientific where
-    (==) = (==) `on` toRational
-    {-# INLINABLE (==) #-}
-
-    (/=) = (/=) `on` toRational
-    {-# INLINABLE (/=) #-}
+    s1 == s2 = c1 == c2 && e1 == e2
+      where
+        Scientific c1 e1 = normalize s1
+        Scientific c2 e2 = normalize s2
 
+-- | Scientific numbers can be safely compared for ordering. No magnitude @10^e@
+-- is calculated so there's no risk of a blowup in space or time when comparing
+-- scientific numbers coming from untrusted sources.
 instance Ord Scientific where
-    (<) = (<) `on` toRational
-    {-# INLINABLE (<) #-}
+    compare s1 s2
+        | c1 == c2 && e1 == e2 = EQ
+        | c1 < 0    = if c2 < 0 then cmp (-c2) e2 (-c1) e1 else LT
+        | c1 > 0    = if c2 > 0 then cmp   c1  e1   c2  e2 else GT
+        | otherwise = if c2 > 0 then LT else GT
+      where
+        Scientific c1 e1 = normalize s1
+        Scientific c2 e2 = normalize s2
 
-    (<=) = (<=) `on` toRational
-    {-# INLINABLE (<=) #-}
+        cmp cx ex cy ey
+            | log10sx < log10sy = LT
+            | log10sx > log10sy = GT
+            | d < 0     = if cx <= (cy `quotInteger` magnitude (-d)) then LT else GT
+            | d > 0     = if cy >  (cx `quotInteger` magnitude   d)  then LT else GT
+            | otherwise = if cx < cy                                 then LT else GT
+          where
+            log10sx = log10cx + ex
+            log10sy = log10cy + ey
 
-    (>) = (>) `on` toRational
-    {-# INLINABLE (>) #-}
+            log10cx = integerLog10' cx
+            log10cy = integerLog10' cy
 
-    (>=) = (>=) `on` toRational
-    {-# INLINABLE (>=) #-}
-
-    compare = compare `on` toRational
-    {-# INLINABLE compare #-}
+            d = log10cx - log10cy
 
+-- | /WARNING:/ '+' and '-' compute the 'Integer' magnitude: @10^e@ where @e@ is
+-- the difference between the @'base10Exponent's@ of the arguments. If these
+-- methods are applied to arguments which have huge exponents this could fill up
+-- all space and crash your program! So don't apply these methods to scientific
+-- numbers coming from untrusted sources. The other methods can be used safely.
 instance Num Scientific where
     Scientific c1 e1 + Scientific c2 e2
        | e1 < e2   = Scientific (c1   + c2*l) e1
@@ -264,12 +292,12 @@ instance Real Scientific where
   "realToFrac_toRealFloat_Float"
    realToFrac = toRealFloat :: Scientific -> Float #-}
 
--- | /WARNING:/ 'recip' and '/' will diverge (i.e. loop and consume all space)
--- when their outputs are <https://en.wikipedia.org/wiki/Repeating_decimal repeating decimals>.
+-- | /WARNING:/ 'recip' and '/' will throw an error when their outputs are
+-- <https://en.wikipedia.org/wiki/Repeating_decimal repeating decimals>.
 --
--- 'fromRational' will diverge when the input 'Rational' is a repeating decimal.
--- Consider using 'fromRationalRepetend' for these rationals which will detect
--- the repetition and indicate where it starts.
+-- 'fromRational' will throw an error when the input 'Rational' is a repeating
+-- decimal.  Consider using 'fromRationalRepetend' for these rationals which
+-- will detect the repetition and indicate where it starts.
 instance Fractional Scientific where
     recip = fromRational . recip . toRational
     {-# INLINABLE recip #-}
@@ -277,23 +305,45 @@ instance Fractional Scientific where
     x / y = fromRational $ toRational x / toRational y
     {-# INLINABLE (/) #-}
 
-    fromRational rational
-        | d == 0    = throw DivideByZero
-        | otherwise = positivize (longDiv 0 0) (numerator rational)
+    fromRational rational =
+        case mbRepetendIx of
+          Nothing -> s
+          Just _ix -> error $
+            "fromRational has been applied to a repeating decimal " ++
+            "which can't be represented as a Scientific! " ++
+            "It's better to avoid performing fractional operations on Scientifics " ++
+            "and convert them to other fractional types like Double as early as possible."
       where
-        -- Divide the numerator by the denominator using long division.
-        longDiv :: Integer -> Int -> (Integer -> Scientific)
-        longDiv !c !e  0 = Scientific c e
-        longDiv !c !e !n
-                          -- TODO: Use a logarithm here!
-            | n < d     = longDiv (c * 10) (e - 1) (n * 10)
-            | otherwise = case n `quotRemInteger` d of
-                            (#q, r#) -> longDiv (c + q) e r
+        (s, mbRepetendIx) = fromRationalRepetendUnlimited rational
 
-        d = denominator rational
+-- | Although 'fromRational' is unsafe because it will throw errors on
+-- <https://en.wikipedia.org/wiki/Repeating_decimal repeating decimals>,
+-- @unsafeFromRational@ is even more unsafe because it will diverge instead (i.e
+-- loop and consume all space). Though it will be more efficient because it
+-- doesn't need to consume space linear in the number of digits in the resulting
+-- scientific to detect the repetition.
+--
+-- Consider using 'fromRationalRepetend' for these rationals which will detect
+-- the repetition and indicate where it starts.
+unsafeFromRational :: Rational -> Scientific
+unsafeFromRational rational
+    | d == 0    = throw DivideByZero
+    | otherwise = positivize (longDiv 0 0) (numerator rational)
+  where
+    -- Divide the numerator by the denominator using long division.
+    longDiv :: Integer -> Int -> (Integer -> Scientific)
+    longDiv !c !e  0 = Scientific c e
+    longDiv !c !e !n
+                      -- TODO: Use a logarithm here!
+        | n < d     = longDiv (c * 10) (e - 1) (n * 10)
+        | otherwise = case n `quotRemInteger` d of
+                        (#q, r#) -> longDiv (c + q) e r
 
--- | Like 'fromRational', this function converts a `Rational` to a `Scientific`
--- but instead of diverging (i.e loop and consume all space) on
+    d = denominator rational
+
+-- | Like 'fromRational' and 'unsafeFromRational', this function converts a
+-- `Rational` to a `Scientific` but instead of failing or diverging (i.e loop
+-- and consume all space) on
 -- <https://en.wikipedia.org/wiki/Repeating_decimal repeating decimals>
 -- it detects the repeating part, the /repetend/, and returns where it starts.
 --
@@ -335,7 +385,18 @@ fromRationalRepetend
     -> Rational
     -> Either (Scientific, Rational)
               (Scientific, Maybe Int)
-fromRationalRepetend mbLimit rational
+fromRationalRepetend mbLimit rational =
+    case mbLimit of
+      Nothing -> Right $ fromRationalRepetendUnlimited rational
+      Just l  -> fromRationalRepetendLimited l rational
+
+-- | Like 'fromRationalRepetend' but always accepts a limit.
+fromRationalRepetendLimited
+    :: Int -- ^ limit
+    -> Rational
+    -> Either (Scientific, Rational)
+              (Scientific, Maybe Int)
+fromRationalRepetendLimited l rational
         | d == 0    = throw DivideByZero
         | num < 0   = case longDiv (-num) of
                         Left  (s, r)  -> Left  (-s, -r)
@@ -345,11 +406,38 @@ fromRationalRepetend mbLimit rational
         num = numerator rational
 
         longDiv :: Integer -> Either (Scientific, Rational) (Scientific, Maybe Int)
-        longDiv n = case mbLimit of
-                      Nothing -> Right $ longDivNoLimit 0 0 M.empty n
-                      Just l  -> longDivWithLimit (-l) n
+        longDiv = longDivWithLimit 0 0 M.empty
+
+        longDivWithLimit
+            :: Integer
+            -> Int
+            -> M.Map Integer Int
+            -> (Integer -> Either (Scientific, Rational)
+                                  (Scientific, Maybe Int))
+        longDivWithLimit !c !e _ns 0 = Right (Scientific c e, Nothing)
+        longDivWithLimit !c !e  ns !n
+            | Just e' <- M.lookup n ns = Right (Scientific c e, Just (-e'))
+            | e <= (-l) = Left (Scientific c e, n % (d * magnitude (-e)))
+            | n < d = let !ns' = M.insert n e ns
+                      in longDivWithLimit (c * 10) (e - 1) ns' (n * 10)
+            | otherwise = case n `quotRemInteger` d of
+                            (#q, r#) -> longDivWithLimit (c + q) e ns r
+
+        d = denominator rational
+
+-- | Like 'fromRationalRepetend' but doesn't accept a limit.
+fromRationalRepetendUnlimited :: Rational -> (Scientific, Maybe Int)
+fromRationalRepetendUnlimited rational
+        | d == 0    = throw DivideByZero
+        | num < 0   = case longDiv (-num) of
+                        (s, mb) -> (-s, mb)
+        | otherwise = longDiv num
+      where
+        num = numerator rational
+
+        longDiv :: Integer -> (Scientific, Maybe Int)
+        longDiv = longDivNoLimit 0 0 M.empty
 
-        -- Divide the numerator by the denominator using long division.
         longDivNoLimit :: Integer
                        -> Int
                        -> M.Map Integer Int
@@ -362,22 +450,6 @@ fromRationalRepetend mbLimit rational
             | otherwise = case n `quotRemInteger` d of
                             (#q, r#) -> longDivNoLimit (c + q) e ns r
 
-        longDivWithLimit :: Int -> Integer -> Either (Scientific, Rational) (Scientific, Maybe Int)
-        longDivWithLimit l = go 0 0 M.empty
-            where
-              go :: Integer
-                 -> Int
-                 -> M.Map Integer Int
-                 -> (Integer -> Either (Scientific, Rational) (Scientific, Maybe Int))
-              go !c !e _ns 0 = Right (Scientific c e, Nothing)
-              go !c !e  ns !n
-                  | Just e' <- M.lookup n ns = Right (Scientific c e, Just (-e'))
-                  | e <= l    = Left (Scientific c e, n % (d * magnitude (-e)))
-                  | n < d     = let !ns' = M.insert n e ns
-                                in go (c * 10) (e - 1) ns' (n * 10)
-                  | otherwise = case n `quotRemInteger` d of
-                                  (#q, r#) -> go (c + q) e ns r
-
         d = denominator rational
 
 -- |
@@ -393,6 +465,11 @@ fromRationalRepetend mbLimit rational
 --
 -- * @r < -(base10Exponent s)@
 --
+-- /WARNING:/ @toRationalRepetend@ needs to compute the 'Integer' magnitude:
+-- @10^^n@. Where @n@ is based on the 'base10Exponent` of the scientific. If
+-- applied to a huge exponent this could fill up all space and crash your
+-- program! So don't apply this function to untrusted input.
+--
 -- The formula to convert the @Scientific@ @s@
 -- with a repetend starting at index @r@ is described in the paper:
 -- <http://fiziko.bureau42.com/teaching_tidbits/turning_repeating_decimals_into_fractions.pdf turning_repeating_decimals_into_fractions.pdf>
@@ -443,6 +520,10 @@ toRationalRepetend s r
 
     nines = m - 1
 
+-- | /WARNING:/ the methods of the @RealFrac@ instance need to compute the
+-- magnitude @10^e@. If applied to a huge exponent this could take a long
+-- time. Even worse, when the destination type is unbounded (i.e. 'Integer') it
+-- could fill up all space and crash your program!
 instance RealFrac Scientific where
     -- | The function 'properFraction' takes a Scientific number @s@
     -- and returns a pair @(n,f)@ such that @s = n+f@, and:
@@ -566,7 +647,7 @@ whenFloating f s@(Scientific c e)
 -- | Precondition: the 'Scientific' @s@ needs to be an integer:
 -- @base10Exponent (normalize s) >= 0@
 toIntegral :: (Num a) => Scientific -> a
-toIntegral (Scientific c e) = fromInteger c * fromInteger (magnitude e)
+toIntegral (Scientific c e) = fromInteger c * magnitude e
 {-# INLINE toIntegral #-}
 
 
@@ -603,11 +684,11 @@ uninitialised :: error
 uninitialised = error "Data.Scientific: uninitialised element"
 
 -- | @magnitude e == 10 ^ e@
-magnitude :: Int -> Integer
+magnitude :: Num a => Int -> a
 magnitude e | e < maxExpt = cachedPow10 e
             | otherwise   = cachedPow10 hi * 10 ^ (e - hi)
     where
-      cachedPow10 = Primitive.indexArray expts10
+      cachedPow10 = fromInteger . Primitive.indexArray expts10
 
       hi = maxExpt - 1
 
@@ -754,9 +835,20 @@ toBoundedInteger s
 {-# SPECIALIZE toBoundedInteger :: Scientific -> Maybe Word32 #-}
 {-# SPECIALIZE toBoundedInteger :: Scientific -> Maybe Word64 #-}
 
--- | @floatingOrInteger@ determines if the scientific is floating point
--- or integer. In case it's floating-point the scientific is converted
--- to the desired 'RealFloat' using 'toRealFloat'.
+-- | @floatingOrInteger@ determines if the scientific is floating point or
+-- integer.
+--
+-- In case it's floating-point the scientific is converted to the desired
+-- 'RealFloat' using 'toRealFloat' and wrapped in 'Left'.
+--
+-- In case it's integer to scientific is converted to the desired 'Integral' and
+-- wrapped in 'Right'.
+--
+-- /WARNING:/ To convert the scientific to an integral the magnitude @10^e@
+-- needs to be computed. If applied to a huge exponent this could take a long
+-- time. Even worse, when the destination type is unbounded (i.e. 'Integer') it
+-- could fill up all space and crash your program! So don't apply this function
+-- to untrusted input but use 'toBoundedInteger' instead.
 --
 -- Also see: 'isFloating' or 'isInteger'.
 floatingOrInteger :: (RealFloat r, Integral i) => Scientific -> Either r i
@@ -885,6 +977,7 @@ isE c = c == 'e' || c == 'E'
 -- Pretty Printing
 ----------------------------------------------------------------------
 
+-- | See 'formatScientific' if you need more control over the rendering.
 instance Show Scientific where
     show s | coefficient s < 0 = '-':showPositive (-s)
            | otherwise         =     showPositive   s
